@@ -6,6 +6,7 @@
 #include "protocol.h"
 #include "util.h"
 #include "netbase.h"
+#include "main.h"
 
 #ifndef WIN32
 # include <arpa/inet.h>
@@ -16,6 +17,7 @@ static const char* ppszTypeName[] =
     "ERROR",
     "tx",
     "block",
+    "filtered block"
 };
 
 CMessageHeader::CMessageHeader()
@@ -140,6 +142,11 @@ const char* CInv::GetCommand() const
 
 std::string CInv::ToString() const
 {
+    if (type == MSG_BLOCK)
+        return strprintf("%s %s", GetCommand(), BlockHashStr(hash).c_str());
+    if (type == MSG_TX)
+        return strprintf("%s %s", GetCommand(), hash.ToString().substr(0,10).c_str());
+
     return strprintf("%s %s", GetCommand(), hash.ToString().substr(0,20).c_str());
 }
 
