@@ -134,4 +134,13 @@ class CInv
         uint256 hash;
 };
 
+enum
+{
+    MSG_TX = 1,
+    MSG_BLOCK,
+    // Nodes may always request a MSG_FILTERED_BLOCK in a getdata, however,
+    // MSG_FILTERED_BLOCK should not appear in any invs except as a part of getdata.
+    MSG_FILTERED_BLOCK,
+};
+
 #endif // __INCLUDED_PROTOCOL_H__
